/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/

void BibleForm::init () {
	string modulesPath = get_application_dirPath() + "/modules/bibles";

	try {
		m_bibleModules = new BgBible::VanyoG::CBibleModules (modulesPath);
		if (m_bibleModules->count() < 1)
			throw runtime_error (format ("No modules found under %s", modulesPath.c_str()));
		
		for (unsigned translI = 0; translI < m_bibleModules->count(); translI++) {
			string title = m_bibleModules->info(translI)->fullTitle();
			menuTranslations->insertItem (title, translI);
		}
		connect (menuTranslations, SIGNAL(activated(int)), this, SLOT(menuTranslations_activated (int)));

		tabsText = new QTabBar (groupText, "tabsText");
		//QGridLayout * gridText = dynamic_cast<QGridLayout*>(groupText->layout());
		groupTextLayout->remove (textBrowser);
		groupTextLayout->addWidget (tabsText, 0, 0);
		groupTextLayout->addWidget (textBrowser, 1, 0);
		connect (tabsText, SIGNAL(selected(int)), this, SLOT(tabsText_selected(int)));		
		
		m_currentBible = NULL;
		menuTranslations_activated (0);
	} catch (exception & ex) {
		//Greshka; Greshka pri zarejdane na moulite.\nVerojatno instalacijata ne e protekla uspeshno
		QMessageBox::critical (this, "", 
			format ("     (\"%s\").\n     .", ex.what()));
		QApplication::exit (-1);
	}

	textBrowser->setTextFormat( Qt::RichText);
	comboBook_activated (0);
}

void BibleForm::destroy() {
	try {
		delete tabsText;
		delete m_bibleModules;
		
		for (size_t bibleI = 0; bibleI < m_bibles.size(); bibleI++)
			delete m_bibles [bibleI];
	} catch (exception & ex)  {
		//Izhod; OK
		QMessageBox::critical (this, "", ex.what(), "OK");
	}
}

void BibleForm::menuTranslations_activated ( int bibleIndex ) {
	try {
		QString title = menuTranslations->text (bibleIndex);	
		
		if (menuTranslations->isItemChecked (bibleIndex)) {
			int tabI;
			for (tabI = 0; tabI < tabsText->count(); ++tabI ) { 
        		if (title == tabsText->tabAt(tabI)->text()) 
					break;
			}		
		
			const BgBible::VanyoG::CBible * bible = m_bibles [tabI];
			m_bibles.erase (m_bibles.begin() + tabI);
			delete bible;

			menuTranslations->setItemChecked (bibleIndex, false);
		
			if (tabI < tabsText->count()) {
				tabsText->removeTab (tabsText->tabAt (tabI));
				tabsText->repaint();
				
				if (tabI == tabsText->currentTab()) {
					if (tabI > 0 || tabsText->count() > 0) {
						int newTabIndex = tabI > 0 ? tabI - 1 : 0;
						int newTabID = tabsText->tabAt(newTabIndex)->identifier();
						
						tabsText->setCurrentTab (newTabID);
						tabsText_selected (newTabID);					
					} else {
						tabsText->hide();
						textBrowser->hide();
						groupSearch->setEnabled (false);
						toolbarLocation->setEnabled (false);
					}
				}
			}
		} else {
			const BgBible::VanyoG::CBible * bible = m_bibleModules->loadBible (bibleIndex);
			m_bibles.push_back (bible);
			
			if (tabsText->count() == 0) {
				tabsText->show();
				textBrowser->show();
				groupSearch->setEnabled (true);
				toolbarLocation->setEnabled (true);			
			}
			
			menuTranslations->setItemChecked (bibleIndex, true);
			int newTabID = tabsText->addTab (new QTab (title));
			tabsText->repaint();
			
			tabsText->setCurrentTab (newTabID);
			tabsText_selected (newTabID);
		} 
	} catch (exception & ex) {
		//Greshka; Greshka pri zarejdane na moulite.\nVerojatno instalacijata ne e protekla uspeshno
		QMessageBox::critical (this, "", 
			format ("     (\"%s\").\n     .", ex.what()));
	}		
}

void BibleForm::tabsText_selected (int tabID) {
	try {
		int tabIndex = tabsText->indexOf (tabID);
		
		m_currentBibleI = tabIndex;
		m_currentBible = m_bibles [m_currentBibleI];
			
		//load word list
		comboSearchText->clear();
		for (unsigned wordI = 0; wordI < m_currentBible->wordsCount(); wordI++) {
			string word = m_currentBible->getWord (wordI);
			comboSearchText->insertItem (word);
		}
		
		//load book titles
		comboBook->clear();
		for (unsigned bookI = 0; bookI < m_currentBible->booksCount(); bookI++) {
			string title = m_currentBible->titles()->shortTitle(bookI);
			comboBook->insertItem (title);
		}
		
		//restore position
		if (m_currentRef.isNull()) 
			goToRef (BgBible::VanyoG::CRef (0, 0, 0));
		else {
			try {			
				goToRef (m_currentBible->toLocal (m_currentRef));
			} catch (...) {
				goToRef (BgBible::VanyoG::CRef (0, 0, 0));
			}
		}
		
		//restore search
		if (checkRedoSearch->isChecked() && ! m_currentSearchText.isEmpty()) {
			comboSearchText->setCurrentText (m_currentSearchText);
			btnSearch_pressed ();
		}
	} catch (exception & ex) {
		//Greshka; Greshka pri prevkljuchvane na prevodite
		QMessageBox::critical (this, "", ex.what(), "    .");
	}	
}

void BibleForm::comboBook_activated( int bookN )
{
	//QMessageBox::information (this, "alert", "bookCombo_activated()", "OK");
	int chapters = m_currentBible->chaptersCount (unsigned(bookN));
	//QMessageBox::information (this, "chapters=", QString::number(chapters), "OK");
	
	int chapN = comboChapter->currentItem();
	comboChapter->clear();
	for (int i = 1; i <= chapters; i++) {
		comboChapter->insertItem (QString::number(i));
	}
	
	int ch =  (chapN < chapters) ? chapN : chapters - 1;
	comboChapter->setCurrentItem (ch);
	comboChapter_activated (ch);
}

void BibleForm::comboChapter_activated( int chapterN )
{
	//QMessageBox::information (this, "alert", "chapterCombo_activated()", "OK");
	int bookN = comboBook->currentItem();
	int verses = m_currentBible->versesCount (bookN, chapterN);
	//QMessageBox::information (this, "verses=", QString::number(verses), "OK");
	
	int verseN = comboVerse->currentItem();
	comboVerse->clear();
	for (int i = 1; i <= verses; i++) {
		comboVerse->insertItem (QString::number(i));
	}
	
	int v = (verseN < verses) ? verseN : verses - 1;
	comboVerse->setCurrentItem (v);
	comboVerse_activated (v);
}


void BibleForm::comboVerse_activated( int verseN )
{
	//QMessageBox::information (this, "alert", "verseCombo_activated()", "OK");
	int bookN = comboBook->currentItem();
	int chapterN = comboChapter->currentItem();
	goToRef (BgBible::VanyoG::CRef (bookN, chapterN, verseN));
}

void BibleForm::comboSearchText_textChanged( const QString & )
{
}

void BibleForm::comboSearchText_activated( const QString & )
{
	btnSearch_pressed();
}

void BibleForm::checkPartialWord_toggled( bool on)
{
	checkCaseSensitive->setEnabled (on);
}

void BibleForm::btnSearch_pressed()
{
	m_currentSearchText = comboSearchText->currentText();
	if (m_currentSearchText.isEmpty()) {
		//Napomnjane; Molja, vyvdete tekst za tyrsene
		QMessageBox::information (this, "", ",    ", "OK");
		return;
	}

	try {	
		bool wholeWord = ! checkPartialWord->isChecked();
		bool caseSensitive = wholeWord ? false : checkCaseSensitive->isChecked();
		m_currentBible->search (m_currentSearchText, wholeWord, caseSensitive, m_searchResults);
		
		listSearchResults->clear();
		if (m_searchResults.empty())
			//Resultat ot tyrsneto; Teksta ne e nameren v tekustija prevod
			QMessageBox::information (this, "  ", "      ");
		else {
			for (size_t i = 0; i < m_searchResults.size(); i++)
				listSearchResults->insertItem (m_searchResults[i].toString(m_currentBible));
		}
	} catch (exception & ex) {
		QMessageBox::critical (this, "Search Error:", ex.what());
	}
}


void BibleForm::listSearchResults_highlighted(int i)
{
	goToRef(m_searchResults[i]);
}

void BibleForm::fileExit_activated()
{
	this->close();
}

void BibleForm::helpAbout_activated()
{
	//Za programata;
	//Bylgarska Biblija za Linux 0.3 betha
	//Avtor: Radoslav Atanasov, 2005
	//Email: rsdevelop@gmail.comboBook_activated
	//Sys sydeistvieto na www.bibliata.com
	QMessageBox::about (this, " ", 
						"   Linux, 0.3 betha\n"
						":  , 2005\n"
						"Email: rsdevelop@gmail.com\n"
						"(   ,       )\n"
						"\n   www.bibliata.com");
}

void BibleForm::goToRef( const BgBible::VanyoG::CRef & _ref )
{
	try {
		static bool lock = false;
		if (lock) return;
		lock = true;	
	
		BgBible::VanyoG::CRef ref = _ref;
		if (ref.isGlobal())
			ref = m_currentBible->toLocal (ref);
	
		unsigned bookN = ref.bookID();
		if (bookN != (unsigned)comboBook->currentItem()) {
			comboBook->setCurrentItem(bookN);
			comboBook_activated (bookN);
		}
		
		unsigned chapterN = ref.chapter();
		if (chapterN != (unsigned)comboChapter->currentItem()) {
			comboChapter->setCurrentItem (chapterN);
			comboChapter_activated (chapterN);
		}
		
		unsigned verseN = ref.verse ();
		if (verseN != (unsigned)comboVerse->currentItem())
			comboVerse->setCurrentItem (verseN);
			
		m_currentRef = m_currentBible->toGlobal (ref);
		
		QString chapHTML = "";
	
		bool painHyperlinks = viewChapterHyperLinks->isOn();
		QString navHTML = "";
		
		if (painHyperlinks) {
			navHTML += "<p>";
			unsigned chapsCount = m_currentBible->chaptersCount (bookN);
			if (chapterN > 0) //predishna glava
				navHTML += QString("<a name=\"gotoChapter\" href=\"%1\">&lt;&lt;&nbsp; </a> ").arg (chapterN - 1);
				
			for (unsigned chapI = 0; chapI < chapsCount; chapI++) {
				navHTML += QString("<a name=\"gotoChapter\" href=\"%1\">%2</a> ").arg(chapI).arg (chapI+1);
			}
			
			if (chapterN < chapsCount - 1) //sledvasta glava
				navHTML += QString("<a name=\"gotoChapter\" href=\"%1\"> &nbsp;&gt;&gt;</a>").arg (chapterN + 1);
			navHTML += "</p>\n";
			
			chapHTML += navHTML;
		}
		
		unsigned versesCount = m_currentBible->versesCount (bookN, chapterN);
		for (unsigned v = 0; v < versesCount; v++) {
			chapHTML += QString ("<p><a name=\"%1\">").arg (v+1);
			
			string verseText = (* m_currentBible) [BgBible::VanyoG::CRef (bookN, chapterN, v, false)];
			QString verseHTML = QString("[%1] %2").arg(v+1).arg(verseText);
			if (v != verseN) 
				chapHTML += verseHTML;
			else 
				chapHTML += "<font color=\"red\">" + verseHTML + "</font>";
				
			chapHTML += "</a></p>\n";
		}
		
		chapHTML += navHTML;
		
		textBrowser->setText (chapHTML);
		
		unsigned skipNav = painHyperlinks ? 1 : 0; //+1 to skip navigation paragraph
		QRect activeRect = textBrowser->paragraphRect (verseN + skipNav); 
		textBrowser->ensureVisible (activeRect.x(), activeRect.y(), 0, 1000); //1000 is big enough to ensure center
		
		//QMessageBox::information (this, "Alert", "Text changed in BibleForm::goToRef()");
		
		lock = false;
	} catch (exception & ex) {
		QMessageBox::critical (this, "Error", ex.what());
	} catch (...) {
		QMessageBox::critical (this, "Error", "Unknown error");
	}
}

void BibleForm::helpAboutQt_activated()
{
	//Bylgarska Biblija za Linux 0.3 betha
	QMessageBox::aboutQt (this, "   Linux 0.3 betha");
}

void BibleForm::helpWhatsThis_activated()
{
	whatsThis();
}


void BibleForm::_3_betha_demoAction_activated()
{
	try {
		QMessageBox::information (this, "0.3 Betha Demo String", m_currentBible->demoString(), "OK");
	} catch (exception & ex)  {
		QMessageBox::critical (this, "Demo Error", ex.what(), "OK");
	}
}


void BibleForm::btnHideSearch_clicked()
{
	viewSearchBar->toggle();
}

void BibleForm::viewLocationToolbar_toggled( bool on)
{
	if (on)	toolbarLocation->show();
	else	toolbarLocation->hide();
}

void BibleForm::viewSearchBar_toggled( bool on)
{
	if (! on)	
		groupSearch->hide();
	else {
		groupSearch->show();
		comboSearchText->setFocus();
	}
}

void BibleForm::gotoAction_activated()
{
	comboBook->setFocus();
}


void BibleForm::viewFont_activated()
{
	bool ok;
	QFont font = QFontDialog::getFont( &ok, QApplication::font(), this );
	if (ok) {
		QApplication::setFont (font);
		this->setFont (font);
		this->repaint();
	}
}


void BibleForm::textBrowser_anchorClicked( const QString & name, const QString & href)
{
	if (name == "gotoChapter") {
		unsigned bookID = unsigned(comboBook->currentItem ());
		goToRef (BgBible::VanyoG::CRef (bookID, href.toUInt(), 0));		
	}
}

void BibleForm::viewChapterHyperLinks_toggled( bool )
{
	goToRef (m_currentRef);
}

