// Description: 
//
// Author: Radoslav Atanasov <rsdevelop@gmail.com>, (C) 2005
// Copyright: See COPYING file that comes with this distribution

#ifndef FileVector_H
#define FileVector_H

#include <fstream>
using namespace std;

namespace rs {

	template <class RecordT>
	class CFileVector {
		size_t m_size;
		ifstream m_fin;
		RecordT m_bufferedRecord;
	public:
		typedef RecordT record_type;
		enum {RECORD_SIZE = sizeof(RecordT)};
	
		explicit CFileVector (const char* filePath) throw (ios_base::failure) { 
			m_fin.open (filePath, ios_base::binary);
			if (! m_fin)
				throw ios_base::failure (format("CFileVector::CFileVector(): cannot open file %s", filePath));
				
			m_size = m_fin.seekg(0, ios_base::end).tellg();
			m_fin.seekg(0, ios_base::beg);
		}
		
		size_t size() const { 
			return m_size / RECORD_SIZE; 
		} 
		
		const RecordT& operator [] (unsigned recordIndex)  
			throw (out_of_range, ios_base::failure) 
		{
			size_t filePos = recordIndex * RECORD_SIZE;
			if (filePos >= m_size)
				throw out_of_range("CFileVector::operator [](): recordIndex out of range");
			
			m_fin.seekg (filePos, ios_base::beg);
			m_fin.read ((char*) & m_bufferedRecord, RECORD_SIZE);
			
			if ((size_t)m_fin.gcount() < RECORD_SIZE)
				throw ios_base::failure ("CFileVector::operator [](): IO error");
			
			return m_bufferedRecord;
		}
		
		const RecordT& atOffset (size_t offset) 
			throw (out_of_range, ios_base::failure) 
		{
			return (*this) [offset / RECORD_SIZE];
		}
		
		~CFileVector() {
			m_fin.close();
		}
	};
	
}

#endif
