// C++ Interface: vanyog_ref
// Description: 
//
// Author: Radoslav Atanasov <rsdevelop@gmail.com>, (C) 2005
// Copyright: See COPYING file that comes with this distribution

#include <string>
#include <stdexcept>
using namespace std;

namespace BgBible {
	namespace VanyoG {

		struct ITitles {
			enum {MAX_TITLE = 256};
			
			virtual string fullTitle (unsigned localBookID) const throw (out_of_range) = 0;
			virtual string shortTitle (unsigned localBookID) const throw (out_of_range) = 0;
			virtual string abbreviatedTitle (unsigned localBookID) const throw (out_of_range) = 0;
		};
	
		class CRef {
			unsigned m_bookID;
			unsigned m_chapter;
			unsigned m_verse;
			
			enum RefType {refUnknown, refGlobal, refLocal, refNull};
			RefType m_refType;
		public:
			CRef () 
				: m_refType (refNull)
			{
			}
			
			CRef (unsigned bookID, unsigned chapter, unsigned verse)
				: m_bookID(bookID), m_chapter(chapter), 
				m_verse(verse), m_refType(refUnknown)
			{
			}
			
			CRef (unsigned bookID, unsigned chapter, unsigned verse, bool isGlobal)
				: m_bookID(bookID), m_chapter(chapter), 
				m_verse(verse), m_refType(isGlobal ? refGlobal : refLocal)
			{
			}
			
			CRef (const CRef& ref, bool isGlobal)
				: m_bookID(ref.bookID()), m_chapter(ref.chapter()), 
				m_verse(ref.verse()), m_refType(isGlobal ? refGlobal : refLocal)
			{
			}
			
			unsigned bookID () const { return m_bookID; }
			unsigned chapter () const { return m_chapter; }
			unsigned verse () const { return m_verse; }
			
			bool isGlobal () const { return m_refType == refGlobal; }
			bool isLocal () const { return m_refType == refLocal; }
			bool isNull () const { return m_refType == refNull; }
			
			CRef chaptersBeginning () const { return CRef(m_bookID, m_chapter, 1); }
			CRef booksBeginning () const { return CRef(m_bookID, 1, 1); }
			
			bool operator == (const CRef& ref) const {
				return m_bookID == ref.bookID() 
					&& m_chapter == ref.chapter()
					&& m_verse == ref.chapter();
			}
			
			bool operator < (const CRef& ref) const {
				return m_bookID < ref.bookID() 
					|| m_bookID == ref.bookID() && (m_chapter < ref.chapter()
												||	m_chapter == ref.chapter() && m_verse < ref.verse());
			}
			
			string toString (const ITitles * titles) const throw (out_of_range);
			//string toString (const char* format, const ITitles * titles) const throw (out_of_range);
		};

		class CRefDelta {
			int m_bookDelta, m_chapterDelta, m_verseDelta;
		public:
			CRefDelta (int chapterDelta, int verseDelta)
				: m_bookDelta(0), m_chapterDelta(chapterDelta), m_verseDelta(verseDelta)
			{
			}
			
			CRefDelta (int bookDelta, int chapterDelta, int verseDelta)
				: m_bookDelta(bookDelta), m_chapterDelta(chapterDelta), m_verseDelta(verseDelta)
			{
			}
			
			int bookDelta () const { return m_bookDelta; }
			int chapterDelta () const { return m_chapterDelta; }
			int verseDelta () const { return m_verseDelta; }
		};
		
		inline CRef operator + (const CRef& ref, const CRefDelta& delta) {
			return CRef (
				unsigned(ref.bookID() + delta.bookDelta()),
				unsigned(ref.chapter() + delta.chapterDelta()),
				unsigned(ref.verse() + delta.verseDelta()));		
		}
		
		inline CRef operator + (const CRefDelta& delta, const CRef& ref) {
			return ref + delta;
		}
	}
}
